import pytest

from minos.common.configuration.config import MinosConfig
from minos.common.exceptions import MinosConfigException


def test_config_ini_fail():
    with pytest.raises(MinosConfigException):
        instance = MinosConfig(path='./test_fail_config.yaml')

def test_config_rest():
    provider_instance = MinosConfig(path='./tests/test_config.yaml')
    assert provider_instance.rest.broker.host == "localhost"
    assert provider_instance.rest.broker.port == 8900
    assert provider_instance.rest.endpoints[0].name == "AddOrder"


def test_config_events():
    provider_instance = MinosConfig(path='./tests/test_config.yaml')
    assert provider_instance.events.broker.host == "localhost"
    assert provider_instance.events.broker.port == 9092



import collections
import os
import typing as t

import yaml
import abc

from minos.common.exceptions import MinosConfigException

BROKER = collections.namedtuple("Broker", "host port")
ENDPOINT = collections.namedtuple("Endpoint", "name route method controller action")
EVENT = collections.namedtuple("Event", "name controller action")
COMMAND = collections.namedtuple("Command", "name controller action")

EVENTS = collections.namedtuple("Events", "broker items")
COMMANDS = collections.namedtuple("Commands", "broker items")
REST = collections.namedtuple("Rest", "broker endpoints")


class MinosConfigAbstract(abc.ABC):
    __slots__ = "_services", "_path"

    def __init__(self, path: str):
        self._services = {}
        self._path = path
        self._load(path)

    @abc.abstractmethod
    def _load(self, path: str) -> None:
        raise NotImplementedError

    @abc.abstractmethod
    def _get(self, key: str, **kwargs: t.Any):
        raise NotImplementedError

    def _file_exit(self, path: str) -> bool:
        if os.path.isfile(path):
            return True
        return False

    @property
    def services(self) -> t.Dict[str, t.Union[int, str, t.Dict, t.List]]:
        raise NotImplementedError

    @property
    def kafka(self):
        return KAFKA(
            host=self._get("host", "kafka"),
            port=self._get("port", "kafka")
        )


class MinosConfig(MinosConfigAbstract):
    __slots__ = "_data"

    def _load(self, path):
        if self._file_exit(path):
            with open(path) as f:
                self._data = yaml.load(f, Loader=yaml.FullLoader)
        else:
            raise MinosConfigException(f"Check if this path: {path} is correct")

    def _get(self, key: str, **kwargs: t.Any) -> t.Union[str, int, t.Dict[str, t.Any], None]:
        if key in self._data:
            return self._data[key]
        return None

    @property
    def rest(self):
        rest_info = self._get("rest")
        broker = BROKER(host=rest_info['host'], port=rest_info['port'])
        endpoints = []
        for endpoint in rest_info['endpoints']:
            endpoints.append(ENDPOINT(name=endpoint['name'], route=endpoint['route'],
                                      method=endpoint['method'].upper(), controller=endpoint['controller'],
                                      action=endpoint['action'])
                             )
        return REST(broker=broker, endpoints=endpoints)

    @property
    def events(self):
        event_info = self._get("events")
        broker = BROKER(host=event_info['broker'], port=event_info['port'])
        events = []
        for event in event_info['items']:
            events.append(EVENT(name=event['name'], controller=event['controller'],
                                action=event['action'])
                          )
        return EVENTS(broker=broker, items=events)

    @property
    def commands(self):
        command_info = self._get("commands")
        broker = BROKER(host=command_info['broker'], port=command_info['port'])
        commands = []
        for command in command_info['items']:
            commands.append(COMMAND(name=command['name'], controller=command['controller'],
                                    action=command['action'])
                            )
        return COMMANDS(broker=broker, items=commands)



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_namespace_packages

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.md') as history_file:
    history = history_file.read()

requirements = [ 'aiomisc', 'six', 'avro', 'orjson', 'lmdb', 'pyyaml' ]

setup_requirements = ['pytest-runner',]

test_requirements = ['pytest>=3', ]

setup(
    author="Andrea Mucci",
    author_email='andrea@clariteia.com',
    python_requires='>=3.5',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Python Package with common Classes and Utilities used in Minos Microservices",
    install_requires=requirements,
    long_description_content_type='text/markdown',
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='minos_microservice_common',
    name='minos_microservice_common',
    packages=find_namespace_packages(include=['minos.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    version='0.0.1.4-alpha',
    zip_safe=False,
)
